unit addrecform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, ExtDlgs, editform, setupform, procs, LazUTF8, wdb_io, wdb_vars;

type

  { TFAddRec }

  TFAddRec = class(TForm)
    BDoc: TButton;
    Button_ok: TButton;
    Button_cancel: TButton;
    CalendarDialog1: TCalendarDialog;
    DocOpenDialog: TOpenDialog;
    RTime: TEdit;
    RecText: TMemo;
    RDate: TStaticText;
    TrackList: TComboBox;
    LogoList: TComboBox;
    HeaderText: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    LogoImage: TImage;
    procedure BDocClick(Sender: TObject);
    procedure Button_cancelClick(Sender: TObject);
    procedure Button_okClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure LogoListChange(Sender: TObject);
    procedure RDateClick(Sender: TObject);
    procedure RTimeEditingDone(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  FAddRec: TFAddRec;
  RDT: TDateTime;


implementation

{$R *.lfm}

{ TFAddRec }

procedure TFAddRec.FormShow(Sender: TObject);
var
 n: integer;
begin
 LogoList.Items := FEdit.LogosList.Items;
 TrackList.Items.Clear;
 for n:=1 to Tcount do
  TrackList.Items.Add(Tracks[n-1].Name);
 if (LogoList.ItemIndex<0) then LogoList.ItemIndex:=0;
 LogoListChange(FAddRec);
 RDT:=Now;
 RDate.Caption:=FormatDateTime('DD.MM.YYYY',RDT);
 RTime.Text:=FormatDateTime('hh:nn', RDT);
end;

procedure TFAddRec.FormCreate(Sender: TObject);
begin
 FAddRec.Caption:='Добавить новую запись';
 HeaderText.Text:='Новая запись';
 TrackList.Style:=csDropDownList;
 LogoList.Style:=csDropDownList;
 BDoc.Caption:='Загрузить документ';
 RecText.Lines.Clear;
 Button_ok.Caption:='Сохранить';
 Button_cancel.Caption:='Отмена';
end;

procedure TFAddRec.Button_cancelClick(Sender: TObject);
begin
 FAddRec.Hide;
end;

procedure TFAddRec.BDocClick(Sender: TObject);
var
 s: string;
 n: integer;
begin
 DocOpenDialog.Title:='Выберите файлы документов для записи';
 if not DocOpenDialog.Execute then exit;
 for n:=0 to DocOpenDialog.Files.Count-1 do
  begin
   s:=CopyDoc(DocOpenDialog.Files.Strings[n]);
   if (pos('<link>/',s) = 1) then RecText.Lines.Insert(0,s);
  end;
end;

procedure TFAddRec.Button_okClick(Sender: TObject);
var n:integer;
begin
 if (Rcount >= MAX_RECORDS ) then
  begin
   MessageDlg('Ошибка',
    'Превышено максимальное количество записей!', mtError, [mbOk],0 );
   exit;
  end;
 if ( TrackList.ItemIndex < 0 ) then
  begin
   MessageDlg('Ошибка',
    'Не выбрана дорожка!', mtError, [mbOk],0 );
   exit;
  end;
 RLtoRD[Rcount]:=Rcount;
 Records[Rcount].DTime:=RDT;
 Records[Rcount].Logo:=LogoList.ItemIndex;
 Records[Rcount].Track:=TrackList.ItemIndex;
 Records[Rcount].Data.Clear;
 if (length(HeaderText.Text)<1) then
  Records[Rcount].Name:='Запись' else
   Records[Rcount].Name:=HeaderText.Text;
 for n:=0 to ( RecText.Lines.Count - 1 ) do
  Records[Rcount].Data.Add(RecText.Lines.Strings[n]);
 inc(Rcount);
 SortRecDocs(RCount-1);
 FEdit.SortDB;
 FAddRec.Hide;
end;

procedure TFAddRec.FormResize(Sender: TObject);
begin
 HeaderText.Width:=FAddRec.ClientWidth-105;
 RecText.Width:=FAddRec.ClientWidth;
 RecText.Height:=FAddRec.ClientHeight-180;
 Button_ok.Left:=FAddRec.ClientWidth-176;
 Button_cancel.Left:=FAddRec.ClientWidth-86;
 Button_ok.Top:=FAddRec.ClientHeight-25;
 Button_cancel.Top:=Button_ok.Top;
end;

procedure TFAddRec.LogoListChange(Sender: TObject);
begin
 FEdit.LogosImages.GetBitmap
  (FAddRec.LogoList.ItemIndex+2,FAddRec.LogoImage.Picture.Bitmap);
end;

procedure TFAddRec.RDateClick(Sender: TObject);
begin
 CalendarDialog1.Date:=RDT;
 if not CalendarDialog1.Execute then exit;
 RDT:=trunc(CalendarDialog1.Date);
 RDate.Caption:=FormatDateTime('DD.MM.YYYY',RDT);
end;

procedure TFAddRec.RTimeEditingDone(Sender: TObject);
var
 dt: TDateTime;
 h,m: integer;
 s: string;
begin
 dt:=trunc(RDT);
 SplitString(RTime.Text,':');
 h:=0;
 m:=0;
 if (SplitN > 1) then m:=toint(splitA[1]);
 if (SplitN > 0) then h:=toint(splitA[0]);
 if (h<0) then h:=0;
 if (h>23) then h:=23;
 if (m<0) then m:=0;
 if (m>59) then m:=59;
 if (h < 10) then s:='0'+inttostr(h) else s:=inttostr(h);
 if (m < 10) then s:=s+':0'+inttostr(m) else s:=s+':'+inttostr(m);
 RDT:=dt+StrToTime(s);
 RTime.Text:=s;
end;

end.

