unit editform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ExtDlgs, procs, wdb_io, wdb_vars;

type

  { TFEdit }

  TFEdit = class(TForm)
    b_del: TButton;
    b_save: TButton;
    TrackList: TComboBox;
    DateDialog: TCalendarDialog;
    EditTime: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label_Date: TLabel;
    LogosList: TComboBox;
    RecText: TMemo;
    s_bytrack: TButton;
    s_bytime: TButton;
    Label3: TLabel;
    LogoImage: TImage;
    Label2: TLabel;
    LogosImages: TImageList;
    RecHeaderText: TEdit;
    Label1: TLabel;
    RecGroup: TGroupBox;
    RecList: TListBox;
    RecListGroup: TGroupBox;
    procedure b_delClick(Sender: TObject);
    procedure b_saveClick(Sender: TObject);
    procedure EditTimeChange(Sender: TObject);
    procedure EditTimeEditingDone(Sender: TObject);
    procedure EditTimeEnter(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Label6Click(Sender: TObject);
    procedure Label_DateClick(Sender: TObject);
    procedure LogosListChange(Sender: TObject);
    procedure RecHeaderTextChange(Sender: TObject);
    procedure RecHeaderTextEditingDone(Sender: TObject);
    procedure RecListMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RecListSelectionChange(Sender: TObject; User: boolean);
    procedure RecTextChange(Sender: TObject);
    procedure RecTextEditingDone(Sender: TObject);
    procedure s_bytimeClick(Sender: TObject);
    procedure s_bytrackClick(Sender: TObject);
    procedure TrackListChange(Sender: TObject);
    procedure FillForm;
    procedure FillList;
    procedure SortDB;
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  FEdit: TFEdit;
  RLtoRD: array [0..MAX_RECORDS] of longint; //Таблица соответствия списка и базы
  CRec, TRec: WDBrecord;
  CRChanged: boolean;

implementation


{$R *.lfm}

{ TFEdit }

procedure TFEdit.FormCreate(Sender: TObject);
var
 n: integer;
begin
 Rcount:=0;
 FEdit.Caption:='Редактор записей';
 RecListGroup.Caption:='Список записей';
 s_bytime.Caption:='времени';
 s_bytrack.Caption:='дорожкам';
 RecGroup.Caption:='Запись';
 LogosList.Items.Add('Глобус');
 LogosList.Items.Add('Учитель');
 LogosList.Items.Add('Кулак');
 LogosList.Items.Add('Звезда');
 LogosList.Items.Add('Гармонь');
 LogosList.Items.Add('Мегафон');
 LogosList.Items.Add('Россия');
 LogosList.Items.Add('Молния');
 LogosList.Items.Add('Ремонт');
 LogosList.Items.Add('Документ');
 b_save.Caption:='<- Записать';
 b_del.Caption:='Удалить';
 LIndex:=-1;
 for n:=0 to MAX_RECORDS do Records[n].Data := TStringList.Create;
 CRec.Data := TStringList.Create;
 CRec.Data.Add('Содержание записи');
end;

procedure TFEdit.FormDeactivate(Sender: TObject);
begin
 FEdit.Hide;
end;

procedure TFEdit.FormHide(Sender: TObject);
begin
 SaveDB;
end;

procedure TFEdit.b_saveClick(Sender: TObject);
begin
 if ( RecList.ItemIndex < 0 ) then
  //Добавление записи в базу
  begin
   if (Rcount >= MAX_RECORDS ) then
    begin
     MessageDlg('Ошибка',
      'Превышено максимальное количество записей!', mtError, [mbOk],0 );
     exit;
    end;
   RLtoRD[Rcount]:=Rcount;
   Records[Rcount]:=CRec;
   LIndex:=RCount;
   RecList.Items.Add(FormatDateTime('DD.MM - ',Crec.DTime)+CRec.Name);
   RecList.ItemIndex:=RCount;
   inc(Rcount);
   SortRecDocs(RCount-1);
  end else
   begin
    //Сохранение записи в базе
    Records[RLtoRD[RecList.ItemIndex]]:=CRec;
    SortRecDocs(RLtoRD[RecList.ItemIndex]);
    RecList.Items.Strings[RecList.ItemIndex]:=
     FormatDateTime('DD.MM - ',Crec.DTime)+CRec.Name;
   end;
  SortDB;
  FillForm;
  CRChanged:=False;
end;

procedure TFEdit.EditTimeChange(Sender: TObject);
begin
 CRChanged:=True;
end;

procedure TFEdit.b_delClick(Sender: TObject);
var
 n,x:longint;
begin
 if (RecList.ItemIndex < 0) then exit;
 x:=RLtoRD[RecList.ItemIndex];
 if (x < 0) or (x >= Rcount) then exit;
 if (MessageDlg('Удаление записи',
      'Вы действиельно хотите удалить запись "'+
       Records[x].Name+'" ?', mtCustom, mbOkCancel, 0 ) = mrCancel)
        then exit;
 for n:=0 to (Rcount-1) do RLtoRD[n]:=n;
 for n:=x to (Rcount-2) do Records[n]:=Records[n+1];
 dec(Rcount);
 FillList;
 CRChanged:=False;
end;

procedure TFEdit.EditTimeEditingDone(Sender: TObject);
var
 dt: TDateTime;
 h,m: integer;
 s: string;
begin
 dt:=trunc(CRec.DTime);
 SplitString(EditTime.Text,':');
 if (SplitN <> 2) then begin EditTime.Color:=clRed; exit; end;
 h:=toint(splitA[0]);
 m:=toint(splitA[1]);
 if (h<0) then h:=0;
 if (h>23) then h:=23;
 if (m<0) then m:=0;
 if (m>59) then m:=59;
 if (h < 10) then s:='0'+inttostr(h) else s:=inttostr(h);
 if (m < 10) then s:=s+':0'+inttostr(m) else s:=s+':'+inttostr(m);
 CRec.DTime:=dt+StrToTime(s);
 EditTime.Text:=s;
end;

procedure TFEdit.EditTimeEnter(Sender: TObject);
begin
 EditTime.Color:=clWhite;
end;

procedure TFEdit.FormResize(Sender: TObject);
begin
 RecList.Height:=RecListGroup.ClientHeight-44;
 Label3.Top:=RecList.Height;
 s_bytime.Top:=RecListGroup.ClientHeight-25;
 s_bytrack.Top:=s_bytime.Top;
 RecGroup.Width:=FEdit.Width-176;
 RecHeaderText.Width:=RecGroup.Width-12;
 RecText.Height:=RecGroup.Height-240;
end;

procedure TFEdit.FormShow(Sender: TObject);
var
 n: integer;
begin
 LIndex:=-1;
 CRec.DTime:=Now;
 //CRec.Data.Clear;
 //CRec.Data.Add('Содержание записи');
 CRec.Track:=0;
 CRec.Logo:=0;
 for n:=0 to RCount-1 do RLtoRD[n]:=n;
 TrackList.Items.Clear;
 for n:=1 to Tcount do
  TrackList.Items.Add(Tracks[n-1].Name);
 FillForm;
 CRChanged:=False;
 FillList;
end;

procedure TFEdit.Label6Click(Sender: TObject);
begin
 EditTime.Text:=FormatDateTime('hh:nn',Time);
 EditTime.Color:=clWhite;
 EditTime.EditingDone;
end;

procedure TFEdit.Label_DateClick(Sender: TObject);
var
 dt: TDateTime;
begin
 dt:=trunc(Crec.DTime);
 DateDialog.Date:=dt;
 dt:=Crec.DTime-dt;
 DateDialog.Date:=dt;
 if not DateDialog.Execute then exit;
 Crec.DTime:=DateDialog.Date+dt;
 Label_Date.Caption:=FormatDateTime('DD.MM.YYYY',Crec.DTime);
 CRChanged:=True;
end;

procedure TFEdit.LogosListChange(Sender: TObject);
begin
 LogosImages.GetBitmap(LogosList.ItemIndex+2,LogoImage.Picture.Bitmap);
 CRec.Logo:=LogosList.ItemIndex;
 CRChanged:=True;
end;

procedure TFEdit.RecHeaderTextChange(Sender: TObject);
begin
 CRChanged:=True;
end;

procedure TFEdit.RecHeaderTextEditingDone(Sender: TObject);
begin
 CRec.Name:=RecHeaderText.Text;
end;

procedure TFEdit.RecListMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 if (Button <> mbRight) then exit;
 if CRChanged then
  if (MessageDlg('Текущая запись документа не сохранена, при редактировании новой записи все несохранённые данные для текущей будут потеряны. Редактировать новую запись?',
   mtCustom, mbOkCancel, 0 ) = mrCancel) then exit;
 RecList.ItemIndex:=-1;
end;

procedure TFEdit.RecListSelectionChange(Sender: TObject; User: boolean);
begin
 if (RecList.ItemIndex = LIndex) then exit;
 if (RecList.ItemIndex < 0) then
  begin
   b_save.Caption:='<- Записать';
   b_del.Enabled:=False;
   CRChanged:=False;
   LIndex:=-1;
   exit;
  end;
 b_save.Caption:='<- Сохранить';
 b_del.Enabled:=True;
 if CRChanged then
  if (MessageDlg('Текущая запись документа не сохранена, при открытии другой записи все несохранённые данные будут потеряны. Открыть другую запись?',
      mtCustom, mbOkCancel, 0 ) = mrCancel) then
       begin
        RecList.ItemIndex:=LIndex;
        exit;
       end;
 CRec:=Records[RLtoRD[RecList.ItemIndex]];
 FillForm;
 CRChanged:=False;
 LIndex:=RecList.ItemIndex;
end;

procedure TFEdit.RecTextChange(Sender: TObject);
begin
 CRChanged:=True;
end;

procedure TFEdit.RecTextEditingDone(Sender: TObject);
var
 n: integer;
begin
 CRec.Data.Clear;
 for n:=0 to RecText.Lines.Count-1 do
  CRec.Data.Add(RecText.Lines.Strings[n]);
end;

procedure TFEdit.s_bytimeClick(Sender: TObject);
var n: longint;
begin
 if (RecList.ItemIndex > -1) then
 begin
  LIndex:=RLtoRD[RecList.ItemIndex];
  b_save.Click;
 end else LIndex:=-1;
 SortDB;
 for n:=0 to RCount-1 do RLtoRD[n]:=n;
 FillList;
end;

procedure TFEdit.s_bytrackClick(Sender: TObject);
var
 n,i: longint;
 b: boolean;
begin
 if (RCount < 2) then exit;
  if (RecList.ItemIndex > -1) then
  begin
   LIndex:=RLtoRD[RecList.ItemIndex];
   b_save.Click;
  end else LIndex:=-1;
 SortDB;
 b:=true;
 while b do
  begin
   b:=false;
   for n:=0 to (RCount-2) do
    begin
     if (Records[RLtoRD[n]].Track) > (Records[RLtoRD[n+1]].Track) then
      begin
       b:=true;
       i:=RLtoRD[n+1];
       RLtoRD[n+1]:=RLtoRD[n];
       RLtoRD[n]:=i;
      end;
    end;
  end;
 if ( LIndex > -1 ) then
  for n:=0 to (RCount-1) do
   if (RLtoRD[n] = LIndex) then
    begin
     LIndex:=n;
     break;
    end;
 FillList;
end;

procedure TFEdit.TrackListChange(Sender: TObject);
begin
 CRec.Track:=TrackList.ItemIndex;
 CRChanged:=True;
end;

procedure TFEdit.FillForm;
begin
 RecHeaderText.Text:=CRec.Name;
 if (CRec.Logo <= LogosList.Items.Count-1) then
  LogosList.ItemIndex:=CRec.Logo;
 LogosListChange(self);
 TrackList.ItemIndex:=CRec.Track;
 Label_Date.Caption:=FormatDateTime('DD.MM.YYYY',CRec.DTime);
 EditTime.Text:=FormatDateTime('hh:nn',CRec.DTime);
 //RecText.Lines.Clear;
 RecText.Lines := CRec.Data;
 EditTime.Color:=clWhite;
end;

procedure TFEdit.FillList;
var
 n:longint;
begin
 RecList.Items.Clear;
 if (Rcount < 1) then exit;
 for n:=0 to (RCount-1) do
  RecList.Items.Add(FormatDateTime('DD.MM - ',Records[RLtoRD[n]].DTime)
   +Records[RLtoRD[n]].Name);
 if (LIndex < Rcount) and (LIndex > -1) then
  begin
   RecList.ItemIndex:=LIndex;
   CRec:=Records[RLtoRD[RecList.ItemIndex]];
   FillForm;
   CRChanged:=False;
   b_save.Caption:='<- Сохранить';
  end else b_save.Caption:='<- Записать';
end;

procedure TFEdit.SortDB;
//Сортировка базы данных по времени пузырьковым методом
//с сохранением положения записей в списке
var
 n,i,x1,x2: longint;
 b,e: boolean;
begin
 if (RCount < 2) then exit;
 e:=false;
 b:=true;
 while b do
  begin
   b:=false;
   for n:=0 to RCount-2 do
    if (Records[n].DTime > Records[n+1].DTime) then
     begin
      b:=true;
      TRec:=Records[n+1];
      Records[n+1]:=Records[n];
      Records[n]:=TRec;
      x1:=-1;
      x2:=-1;
      for i:=0 to RCount-1 do
       begin
        if (RLtoRD[i] = n) then x1:=i;
        if (RLtoRD[i] = n+1) then x2:=i;
        if (x1 > -1) and (x2 > -1) then break;
       end;
      if (x1=-1) or (x2=-1) then e:=true else
       begin
        RLtoRD[x2]:=n;
        RLtoRD[x1]:=n+1;
       end;
     end;
  end;
 if e then MessageDlg('Ошибка',
    'Ошибка процедуры сортировки!', mtError, [mbOk],0 );
end;

end.

