unit wdb_io;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Dialogs, LazUTF8, Zipper, zlibar, Graphics, procs,
  wdb_vars;

//Сохранение данных дороги
procedure SaveDB;
//Загрузка данных дороги
procedure LoadDB;
//Загрузка данных дороги первой версии
procedure LoadFD1;
//Сохранение дороги нового формата
procedure SaveZFD(FileName: string);
//Загрузка дороги нового формата
procedure LoadZFD(FileName: string);
//Загрузка дороги старого формата
procedure LoadAFD(FileName: string);
//Загрузка документа в рабочую папку дороги
function CopyDoc(DocName: string): string;
//Сохранение настроек
procedure SaveConfig;
//Проверка и загрузка архива
procedure CheckAFD(FileName: string);

implementation

uses MainForm, setupform, editform, logoform, recviewform, addrecform;

procedure SaveDB;
var
 f: textfile;
 n,i: longint;
 s: string;
begin
 assignfile(f,DB_WAYFILE);
 rewrite(f);
 writeln(f,'[WAY CONFIG FILE]');
 writeln(f,'SoftwareVersion='+inttostr(WDBVersionNumber));
 writeln(f,'TrackWidth=',inttostr(round(cfg_tw*1000)));
 writeln(f,'HeaderHeight=',inttostr(round(cfg_hh*1000)));
 writeln(f,'VisiblePart=',inttostr(cfg_wl));
 writeln(f,'BackgroundColor1=',inttostr(RGBtoColor(clr_b11*2,clr_b12*2,clr_b13*2)));
 writeln(f,'BackgroundColor2=',inttostr(RGBtoColor(clr_b21*2,clr_b22*2,clr_b23*2)));
 writeln(f,'HeaderColor1=',inttostr(RGBtoColor(clr_h11*2,clr_h12*2,clr_h13*2)));
 writeln(f,'HeaderColor2=',inttostr(RGBtoColor(clr_h21*2,clr_h22*2,clr_h23*2)));
 writeln(f,'TrackColor1=',inttostr(RGBtoColor(clr_t11*2,clr_t12*2,clr_t13*2)));
 writeln(f,'TrackColor2=',inttostr(RGBtoColor(clr_t21*2,clr_t22*2,clr_t23*2)));
 writeln(f,'TracksCount=',inttostr(Tcount));
 for n:=0 to Tcount-1 do
  begin
   if (Tracks[n].Level<10) then s:='00'+inttostr(Tracks[n].Level)
    else if (Tracks[n].Level<100) then s:='0'+inttostr(Tracks[n].Level)
     else s:=inttostr(Tracks[n].Level);
   if Tracks[n].Colored then s:=s+'1' else s:=s+'0';
   writeln(f,'Track'+inttostr(n)+'Data='+s);
   i:=RGBtoColor(Tracks[n].Color1*2, Tracks[n].Color2*2, Tracks[n].Color3*2);
   writeln(f,'Track'+inttostr(n)+'Color='+inttostr(i));
   writeln(f,'Track'+inttostr(n)+'Name='+Tracks[n].Name);
   writeln(f,'Track'+inttostr(n)+'Hint='+Tracks[n].Hint);
  end;
 closefile(f);
 assignfile(f,DB_DATAFILE);
 rewrite(f);
 writeln(f,'[WAY RECORDS DATA]');
 writeln(f,'RecordsCount='+inttostr(Rcount));
 for n:=0 to Rcount-1 do
  begin
   writeln(f,'<WDB_REC_HEADER>|',Records[n].Track,'|',Records[n].Logo,'|',
    WDBWriteDateTime(Records[n].DTime));
   writeln(f,Records[n].Name);
   for i:=0 to (Records[n].Data.Count-1) do
    //На случай, если в тексте записей есть строки, начинающиеся с заголовка записи
    if (copy(Records[n].Data.Strings[i],1,16) <> '<WDB_REC_HEADER>') then
     writeln(f,Records[n].Data.Strings[i])
      else writeln(f,' '+Records[n].Data.Strings[i]);
  end;
 closefile(f);
end;

procedure LoadDB;
var
 f: textfile;
 s: string;
 n: longint;
 e: boolean;
 c: TColor;
begin
 e:=false;
 if not fileexists(DB_WAYFILE) then
  begin
   MessageDlg('Ошибка','Файл дороги не найден в рабочем окружении!',
    mtError, [mbOk],0 );
   LogoLock:=False;
   exit;
  end;
 //Загрузка файла дороги
 assignfile(f,DB_WAYFILE);
 reset(f);
 readln(f,s);
 if (s <> '[WAY CONFIG FILE]') then e:=true;
 closefile(f);
 if (toint(getpar(DB_WAYFILE,'SoftwareVersion')) > WDBVersionNumber) then
  MessageDlg('Предупреждение','Документ создан в более новой версии программы.',
   mtInformation, [mbOk],0 );
 cfg_tw:=toint(getpar(DB_WAYFILE,'TrackWidth')) / 1000;
 cfg_hh:=toint(getpar(DB_WAYFILE,'HeaderHeight')) / 1000;
 cfg_wl:=toint(getpar(DB_WAYFILE,'VisiblePart'));
 SetWLT;
 c:=toint(getpar(DB_WAYFILE,'BackgroundColor1'));
 clr_b11:=Red(c) div 2;
 clr_b12:=Green(c) div 2;
 clr_b13:=Blue(c) div 2;
 c:=toint(getpar(DB_WAYFILE,'BackgroundColor2'));
 clr_b21:=Red(c) div 2;
 clr_b22:=Green(c) div 2;
 clr_b23:=Blue(c) div 2;
 c:=toint(getpar(DB_WAYFILE,'HeaderColor1'));
 clr_h11:=Red(c) div 2;
 clr_h12:=Green(c) div 2;
 clr_h13:=Blue(c) div 2;
 c:=toint(getpar(DB_WAYFILE,'HeaderColor2'));
 clr_h21:=Red(c) div 2;
 clr_h22:=Green(c) div 2;
 clr_h23:=Blue(c) div 2;
 c:=toint(getpar(DB_WAYFILE,'TrackColor1'));
 clr_t11:=Red(c) div 2;
 clr_t12:=Green(c) div 2;
 clr_t13:=Blue(c) div 2;
 c:=toint(getpar(DB_WAYFILE,'TrackColor2'));
 clr_t21:=Red(c) div 2;
 clr_t22:=Green(c) div 2;
 clr_t23:=Blue(c) div 2;
 TCount:=toint(getpar(DB_WAYFILE,'TracksCount'));
 if (TCount > MAX_TRACKS) then TCount := MAX_TRACKS;
 if (TCount < 0) then TCount := 0;
 for n:=0 to Tcount-1 do
  begin
   s:=getpar(DB_WAYFILE,'Track'+inttostr(n)+'Data');
   Tracks[n].Level:=toint(copy(s,1,3));
   if (Tracks[n].Level > 19) then Tracks[n].Level:=19;
   if (Tracks[n].Level > 9) then Tracks[n].Level:=Tracks[n].Level - 10;
   if (s[4] = '0') then Tracks[n].Colored:=False else Tracks[n].Colored:=True;
   s:=getpar(DB_WAYFILE,'Track'+inttostr(n)+'Color');
   c:=toint(s);
   Tracks[n].Color1 := Red(c) div 2;
   Tracks[n].Color2 := Green(c) div 2;
   Tracks[n].Color3 := Blue(c) div 2;
   Tracks[n].Name:=getpar(DB_WAYFILE,'Track'+inttostr(n)+'Name');
   Tracks[n].Hint:=getpar(DB_WAYFILE,'Track'+inttostr(n)+'Hint');
  end;
 //Очищаем незаполненные данные
 for n:=TCount to MAX_TRACKS do
  begin
   Tracks[n].Level:=0;
   Tracks[n].Colored:=False;
   Tracks[n].Color1 := 10;
   Tracks[n].Color2 := 10;
   Tracks[n].Color3 := 10;
   Tracks[n].Name:='Дорожка';
   Tracks[n].Hint:='';
  end;
 //Загрузка файла записей
 RCount:=0;
 if not fileexists(DB_DATAFILE) then
  begin
   MessageDlg('Ошибка','Файл данных не найден в рабочем окружении!',
    mtError, [mbOk],0 );
   LogoLock:=False;
   exit;
  end;
 assignfile(f,DB_DATAFILE);
 reset(f);
 readln(f,s);
 if (s <> '[WAY RECORDS DATA]') then e:=true;
 readln(f,s);
 SPlitString(s,'=');
 if (SplitN <> 2) or (SplitA[0] <> 'RecordsCount') then e:=true;
 RCount:=toint(SplitA[1]);
 if (RCount < 0) then RCount:=0;
 if (RCount > MAX_RECORDS) then RCount := MAX_RECORDS;
 n:=-1;
 while not eof(f) do
  begin
   readln(f,s);
   if (copy(s,1,16) = '<WDB_REC_HEADER>') then
    begin
     SplitString(s,'|');
     if (SplitN = 4) then
      begin
       inc(n);
       Records[n].Track:=toint(SplitA[1]);
       Records[n].Logo:=toint(SplitA[2]);
       Records[n].DTime:=WDBReadDateTime(SplitA[3]);
       if not eof(f) then
        begin
         readln(f,s);
         Records[n].Name:=s;
        end else e:=true;
       Records[n].Data.Clear;
      end;
    end else
     if (n>-1) then Records[n].Data.Add(s);
  end;
 inc(n);
 closefile(f);
 if (RCount <> n) then
  begin
   MessageDlg('Предупреждение',
    'Файл данных содержал неверную информацию о количестве записей ('+
     inttostr(RCount)+' <> '+inttostr(n)+')', mtInformation, [mbOk],0 );
   RCount:=n;
   e:=true;
  end;
 if e then MessageDlg('Ошибка',
  'Во время загрузки файлов данных произошли ошибки.',mtError, [mbOk],0 );
end;

procedure LoadFD1;
var
 f: textfile;
 s: string;
 n: longint;
 i,tc: integer;
 e: boolean;
begin
 e:=false;
 if not fileexists('waydata.fd') then
  begin
   MessageDlg('Ошибка','Файл дороги не найден в рабочем окружении!',
    mtError, [mbOk],0 );
   LogoLock:=False;
   exit;
  end;
 assignfile(f,'waydata.fd');
 reset(f);
 readln(f,s);
 if (s <> 'WAY DB FILE') then e:=true;
 readln(f,s);
 n:=length(s);
 if (n < 10) then e:=true;
 readln(f,s);
 n:=toInt(s);
 //if (n > WDBVersionNumber) then MessageDlg('Предупреждение',
 // 'Документ создан в более новой версии программы.',mtInformation, [mbOk],0 );
 readln(f,s);
 if (s <> 'CONFIG') then e:=true;
 if e then
  begin
   MessageDlg('Ошибка',
    'Ошибка в заголовке рабочего документа!',mtError, [mbOk],0 );
   closefile(f);
   exit;
  end;
 readln(f,s);
 SplitString(s,';');
 if (SplitN<>3) then e:=true;
 cfg_tw:=toDouble(SplitA[0]);
 cfg_hh:=toDouble(SplitA[1]);
 cfg_wl:=toInt(SplitA[2]);
 SetWLT;
 readln(f,s);
 SplitString(s,';');
 if (SplitN<>6) then e:=true;
 clr_b11:=toInt(SplitA[0]);
 clr_b12:=toInt(SplitA[1]);
 clr_b13:=toInt(SplitA[2]);
 clr_b21:=toInt(SplitA[3]);
 clr_b22:=toInt(SplitA[4]);
 clr_b23:=toInt(SplitA[5]);
 readln(f,s);
 SplitString(s,';');
 if (SplitN<>6) then e:=true;
 clr_h11:=toInt(SplitA[0]);
 clr_h12:=toInt(SplitA[1]);
 clr_h13:=toInt(SplitA[2]);
 clr_h21:=toInt(SplitA[3]);
 clr_h22:=toInt(SplitA[4]);
 clr_h23:=toInt(SplitA[5]);
 readln(f,s);
 SplitString(s,';');
 if (SplitN<>6) then e:=true;
 clr_t11:=toInt(SplitA[0]);
 clr_t12:=toInt(SplitA[1]);
 clr_t13:=toInt(SplitA[2]);
 clr_t21:=toInt(SplitA[3]);
 clr_t22:=toInt(SplitA[4]);
 clr_t23:=toInt(SplitA[5]);
 readln(f,s); readln(f,s); readln(f,s); readln(f,s); readln(f,s);
 readln(f,s);
 if (s <> 'TRACKS') then e:=true;
 if e then
  begin
   MessageDlg('Ошибка',
    'Ошибка в параметрах рабочего документа!',mtError, [mbOk],0 );
   closefile(f);
   exit;
  end;
 readln(f,s);
 n:=toInt(s);
 if (n<0) or (n>MAX_TRACKS) then e:=true
  else Tcount:=n;
 for n:=0 to Tcount-1 do
  begin
   readln(f,s);
   if (toInt(s) <> n+1) then e:=true;
   readln(f,Tracks[n].Name);
   readln(f,Tracks[n].Hint);
   readln(f,s);
   if (length(s) < 2) then e:=true;
   if (toInt(s[1]) = 1) then Tracks[n].Colored:=true
    else Tracks[n].Colored:=false;
   Tracks[n].Level:=toInt(s[2]);
   readln(f,s);
   SplitString(s,';');
   if (SplitN<>3) then e:=true;
   Tracks[n].Color1:=toInt(SplitA[0]);
   Tracks[n].Color2:=toInt(SplitA[1]);
   Tracks[n].Color3:=toInt(SplitA[2]);
  end;
 readln(f,s);
 if (s <> 'RECORDS') then e:=true;
 if e then
  begin
   MessageDlg('Ошибка',
    'Ошибка в описании дорожек рабочего документа!',mtError, [mbOk],0 );
   closefile(f);
   exit;
  end;
 readln(f,s);
 n:=toInt(s);
 if (n<0) or (n>MAX_RECORDS) then e:=true
  else Rcount:=n;
 for n:=0 to Rcount-1 do
  begin
   readln(f,s);
   if (s<>'WDB RECORD') then e:=true;
   readln(f,s);
   if (toInt(s) <> n+1) then e:=true;
   readln(f,Records[n].Name);
   readln(f,Records[n].DTime);
   readln(f,s);
   SplitString(s,';');
   if (SplitN<>5) then e:=true;
   Records[n].Logo:=toInt(SplitA[0]);
   Records[n].Track:=toInt(SplitA[1]);
   Records[n].Data.Clear;
   if (toInt(SplitA[2]) > -1) then
    begin
     s:=SplitA[2]+'.'+SplitA[3];
     Records[n].Data.Add('<link>/'+s+'/'+s);
    end;
   tc:=toInt(SplitA[4]);
   if (tc < 0) then tc:=0;
   for i:=0 to tc-1 do
    begin
     readln(f,s);
     Records[n].Data.Add(s);
     if eof(f) then begin e:=true; break; end;
    end;
   RLtoRD[n]:=n;
  end;
 if e then
  begin
   MessageDlg('Ошибка',
    'Ошибка в записях рабочего документа!',mtError, [mbOk],0 );
   closefile(f);
   exit;
  end;
 readln(f,s);
 if (s <> 'END OF FILE') then e:=true;
 if e then MessageDlg('Ошибка',
  'Во время загрузки рабочего документа произошли ошибки.',mtError, [mbOk],0 );
 closefile(f);
 //Сохраняем данные в новом формате
 SaveDB;
 //Удаляем старый файл
 deletefile('waydata.fd');
end;

procedure SaveZFD(FileName: string);
var
 zipfd: TZipper;
 n,ec,i: longint;
 df: string;
begin
 LogoLock:=True;
 ec:=0;
 FLogo.Label2.Caption:='Сохранение архива дороги...';
 FLogo.Label3.Caption:='Сохранение файла записей...';
 FLogo.Visible:=True;
 Application.ProcessMessages;
 SaveDB;
 if not fileexists(DB_DATAFILE) or not fileexists(DB_WAYFILE) then
  begin
   MessageDlg('Ошибка',
    'При сохранении базы данных произошла ошибка! [0]', mtError, [mbOk],0 );
   LogoLock:=False;
   FLogo.Hide;
   exit;
  end;
 if not directoryexists(DB_FILESDIR) then mkdir (DB_FILESDIR);
 FLogo.Label3.Caption:='Подготовка архива...';
 Application.ProcessMessages;
 zipfd := TZipper.Create;
 zipfd.FileName := UTF8toSys(FileName);
 zipfd.Entries.AddFileEntry(DB_WAYFILE);
 zipfd.Entries.AddFileEntry(DB_DATAFILE);
 zipfd.Entries.AddFileEntry(DB_FILESDIR);
 for n:=0 to Rcount-1 do
  begin
   FLogo.Label3.Caption:='Сохранение документов... '
    +inttostr(round(100*n/Rcount))+'%';
   Application.ProcessMessages;
   for i:=0 to Records[n].Data.Count-1 do
    begin
     if (pos('<link>/',Records[n].Data.Strings[i]) <> 1) then break;
     SplitString(Records[n].Data.Strings[i],'/');
     if (SplitN <> 3) then break;
     df:=DB_FILESDIR+DIR_SPLITTER+SplitA[2];
     if not fileexists(df) then inc(ec) else zipfd.Entries.AddFileEntry(df,df);
    end;
  end;
 FLogo.Label3.Caption:='Сохранение архива...';
 Application.ProcessMessages;
 zipfd.ZipAllFiles;
 zipfd.Free;
 db_filename:=FileName;
 MForm.Caption:=db_filename + ' - Дорога v. '+WDBVersion;
 LogoLock:=False;
 FLogo.Hide;
 if (ec > 0) then MessageDlg('Ошибка',
  'При сохранении базы данных произошли ошибки! [ ' +
   inttostr(ec)+ ' ]', mtError, [mbOk],0 );
end;

procedure LoadZFD(FileName: string);
var
 n,i: longint;
 e: boolean;
 zipfd: TUnZipper;
 fl: TStringList;
begin
 if not fileexists(UTF8toSys(FileName)) then
  begin
   MessageDlg('Ошибка',
    'Не найден файл "'+FileName+'" !', mtError, [mbOk],0 );
   exit;
  end;
 e:=false;
 FLogo.Label2.Caption:='Загрузка архива дороги...';
 FLogo.Label3.Caption:='Открытие архива...';
 FLogo.Show;
 Application.ProcessMessages;
 LogoLock:=True;
 zipfd:= TUnZipper.Create;
 zipfd.FileName:=UTF8toSys(FileName);
 zipfd.OutputPath:='';
 zipfd.Examine;
 i:=0;
 for n:=0 to zipfd.Entries.Count-1 do
  begin
   if (zipfd.Entries.Entries[n].ArchiveFileName = DB_WAYFILE) then inc(i);
   if (zipfd.Entries.Entries[n].ArchiveFileName = DB_DATAFILE) then inc(i);
  end;
 if (i<2) then e:=true;
 FLogo.Label3.Caption:='Распаковка данных...';
 Application.ProcessMessages;
 fl:=TStringList.Create;
 fl.Add(DB_WAYFILE);
 fl.Add(DB_DATAFILE);
 fl.Add(DB_FILESDIR);
 zipfd.UnZipFiles(fl);
 FLogo.Label3.Caption:='Распаковка документов...';
 Application.ProcessMessages;
 fl.Clear;
 for n:=0 to zipfd.Entries.Count-1 do
  if (pos(DB_FILESDIR, zipfd.Entries.Entries[n].ArchiveFileName) = 1) then
   fl.Add(zipfd.Entries.Entries[n].ArchiveFileName);
 if (fl.Count > 0) then zipfd.UnZipFiles(fl);
 zipfd.Free;
 fl.Free;
 LoadDB;
 LogoLock:=False;
 FLogo.Hide;
 db_filename:=FileName;
 MForm.Caption:=db_filename + ' - Дорога v. '+WDBVersion;
 SetupChanged:=True;
 Application.ProcessMessages;
 if e then MessageDlg('Ошибка', 'При открытии файла произошли ошибки!',
  mtError, [mbOk],0 );
end;

procedure LoadAFD(FileName: string);
var
 n: longint;
 RZar: TZlibReadArchive;
 ZStream: TMemoryStream;
 Stream: TMemoryStream;
begin
 if not fileexists(UTF8toSys(FileName)) then
  begin
   MessageDlg('Ошибка',
    'Не найден файл "'+FileName+'" !', mtError, [mbOk],0 );
   exit;
  end;
 LogoLock:=True;
 FLogo.Label2.Caption:='Загрузка архива дороги...';
 FLogo.Label3.Caption:='Загрузка...';
 FLogo.Show;
 Application.ProcessMessages;
 if not directoryexists(DB_FILESDIR) then mkdir (DB_FILESDIR);
 //showmessage(FileName);
 ZStream := TMemoryStream.Create;
 Stream := TMemoryStream.Create;
 ZStream.LoadFromFile(UTF8toSys(FileName));
 FLogo.Label3.Caption:='Открытие архива...';
 Application.ProcessMessages;
 RZar:= TZlibReadArchive.Create(ZStream);
 if (RZar.Count<1) then
  begin
   MessageDlg('Ошибка',
    'Ошибка архива "'+FileName+'" [0]', mtError, [mbOk],0 );
   RZar.Free;
   ZStream.Free;
   Stream.Free;
   LogoLock:=False;
   FLogo.Hide;
   exit;
  end;
 FLogo.Label3.Caption:='Загрузка базы данных...';
 Application.ProcessMessages;
 RZar.ExtractFileToStream(0,Stream);
 Stream.SaveToFile('waydata.fd');
 for n:=1 to (RZar.Count-1) do
  begin
   FLogo.Label3.Caption:='Загрузка документов... ( '+inttostr(n)+' / '+
    inttostr(RZar.Count-1)+' )';
   Application.ProcessMessages;
   RZar.ExtractFileToStream(n,Stream);
   Stream.SaveToFile(DB_FILESDIR+DIR_SPLITTER+RZar.FilesInArchive[n].FileName);
  end;
 RZar.Free;
 ZStream.Free;
 Stream.Free;
 LoadFD1;
 db_filename:=FileName;
 MForm.Caption:=db_filename + ' - Дорога v. '+WDBVersion;
 LogoLock:=False;
 FLogo.Hide;
 SetupChanged:=True;
end;

function CopyDoc(DocName: string): string;
var
 buf: array [0..8191] of byte;
 sf,df: file;
 dn,s: string;
 x: integer;
 fs,n: int64;
begin
 CopyDoc:='error';
 if (length(DocName) < 2) then exit;
 if not directoryexists(DB_FILESDIR) then mkdir (DB_FILESDIR);
 SplitString(DocName,DIR_SPLITTER);
 s:=SplitA[SplitN-1];
 if fileexists(UTF8toSys(DB_FILESDIR+DIR_SPLITTER+s)) then
  begin
   SplitString(s,'.');
   dn:=SplitA[SplitN-1];
   for x:=0 to 65535 do
    if not fileexists(UTF8toSys(DB_FILESDIR+DIR_SPLITTER+inttostr(x)+'.'+dn)) then
     begin
      dn:=inttostr(x)+'.'+dn;
      break;
     end;
  end else dn:=s;
 if fileexists(UTF8toSys(DB_FILESDIR+DIR_SPLITTER+dn)) then exit;
 LogoLock:=True;
 FLogo.visible:=True;
 FLogo.Label2.Caption:='Загрузка документа: 0 %';
 FLogo.Label3.Caption:=DocName;
 Application.ProcessMessages;
 {$I-}
 FileMode:=0;
 assignfile(sf,UTF8toSys(DocName));
 assignfile(df,UTF8toSys(DB_FILESDIR+DIR_SPLITTER+dn));
 reset(sf,1);
 rewrite(df,1);
 fs:=filesize(sf);
 if IOResult<>0 then fs:=-1;
 n:=0;
 if (fs > 0) then
  while true do
   begin
    blockread(sf,buf,8192,x);
    FileMode:=2;
    if (x<1) then break
     else blockwrite(df,buf,x);
    n:=n+x;
    FLogo.Label2.Caption:='Загрузка документа: '+inttostr(round(100*n/fs))+' %';
    Application.ProcessMessages;
    FileMode:=0;
   end;
 closefile(sf);
 closefile(df);
 FileMode:=2;
 {$I+}
 if (fs > -1) then CopyDoc:='<link>/'+s+'/'+dn;
 LogoLock:=False;
 FLogo.Hide;
end;

procedure SaveConfig;
var
 f: textfile;
 n: integer;
begin
 assignfile(f,CFG_FILE);
 rewrite(f);
 writeln(f,'[Настройки ППД]');
 writeln(f,'Version='+WDBVersion);
 writeln(f,'MFormLeft='+inttostr(MForm.Left));
 writeln(f,'MFormTop='+inttostr(MForm.Top));
 writeln(f,'MFormWidth='+inttostr(MForm.Width));
 writeln(f,'MFormHeight='+inttostr(MForm.Height));
 writeln(f,'FEditLeft='+inttostr(FEdit.Left));
 writeln(f,'FEditTop='+inttostr(FEdit.Top));
 writeln(f,'FEditWidth='+inttostr(FEdit.Width));
 writeln(f,'FEditHeight='+inttostr(FEdit.Height));
 writeln(f,'FRecViewLeft='+inttostr(FRecView.Left));
 writeln(f,'FRecViewTop='+inttostr(FRecView.Top));
 writeln(f,'FRecViewWidth='+inttostr(FRecView.Width));
 writeln(f,'FRecViewHeight='+inttostr(FRecView.Height));
 writeln(f,'FSetupLeft='+inttostr(FSetup.Left));
 writeln(f,'FSetupTop='+inttostr(FSetup.Top));
 writeln(f,'FAddRecLeft='+inttostr(FAddRec.Left));
 writeln(f,'FAddRecTop='+inttostr(FAddRec.Top));
 writeln(f,'FAddRecWidth='+inttostr(FAddRec.Width));
 writeln(f,'FAddRecHeight='+inttostr(FAddRec.Height));
 for n:=0 to Doc_count-1 do
  begin
   writeln(f,'Doc'+inttostr(n)+'ext='+Doc_ext[n]);
   writeln(f,'Doc'+inttostr(n)+'cmd='+Doc_cmd[n]);
  end;
 closefile(f);
end;

procedure CheckAFD(FileName: string);
var
 r: byte;
 b: array [0..1] of char;
 f: file;
begin
r:=0;
b:='  ';
{$I-}
assignfile(f,UTF8toSys(FileName));
reset(f,1);
blockread(f,b,2);
closefile(f);
IOResult;
{$I+}
if (b='ZA') then r:=1;
if (b='PK') then r:=2;
if (r=1) then LoadAFD(UTF8toSys(FileName));
if (r=2) then LoadZFD(UTF8toSys(FileName));
if (r=0) then MessageDlg('Ошибка загрузки документа',
  'Не удалось определить тип файла "'+FileName+'"', mtError, [mbOk], 0 );
end;

end.

